/*
 engine.cpp

 Frode Aarstad.
*/

#include "engine.h"


void engine::myDraw()
{
  glMultMatrixf(mNavMatrix.getFloatPtr());

  // -------------------- THE DIFFERENT STATES ------------------------

  if (str == "idle" && menuMode == 0) {
    // we are just walking around
  }

  else if (str == "draw") {
    // we are in the drawing mode, wall drawing mode.. 
    glDisable(GL_LIGHTING);
    myMazeDrawer->addWall();
    myMazeDrawer->draw();
    glEnable(GL_LIGHTING);
  } 
  
  //---------------------- DRAW PHASE -----------------------------

  // draw the bottom grid.
  glPushMatrix();
  glCallList(grid_list);
  glPopMatrix();

  
  // if the myMaze3D object has been initialized show it..
  if (myMaze3D != 0x0) {
    if (selected_triangle != 99) myMaze3D->draw(selected_triangle);
    else myMaze3D->draw(99);
  }

  if (sim2 != 0x0) {
    if (show_path) sim2->drawPath(Path2, "red");
    if (show_nodes) sim2->drawNodes("yellow");	
    if (animate_path) sim2->simulatePath(Path2, theFlyingRobot);
  }

  theRollingRobot->draw(0);
  theFlyingRobot->draw(0);
  theGoal->draw(0);


  // --------------------------- WAND STUFF ---------------------------------

  glEnable(GL_DEPTH_TEST);
  // drawing the appropriate wand icon
  if (menuMode== 0) 
    drawWand(wandMatrix, str);
  else 
    drawWand(wandMatrix, "something blabla"); 

  // --------------------------- MENU ---------------------------------

  if (menuMode == 1) {
    myMenu->draw(headMatrix);
    myMenu->selected(wandMatrix, headMatrix);
    myMenu->drawSubmenu(str, headMatrix);
    velocity = 0.0f;
  }

  if (menuMode == 0) glEnable(GL_DEPTH_TEST);
  else glDisable(GL_DEPTH_TEST);

}  // end myDraw();




void engine::drawWand(vjMatrix *wand, string i) 
{
  // draw a different icon depending upon the
  // current state. 

  float x,y,z;
  wand->getTrans(x, y, z);

  if (i == "draw") {
    // we are in the sketching mode.

    // adding constraints to painting area
    if (x > WALL_SIZE) x = (float)WALL_SIZE;
    else if (x < 0) x = 0;
    if (y > WALL_SIZE) y = (float)WALL_SIZE;
    else if (y < 0) y = 0;
  
    glPushMatrix(); 
      glLoadIdentity();
      vjMatrix *tmp;
      tmp = wandMatrix;
      tmp->setTrans(x,y, 0.0);
      glMultMatrixf(tmp->getFloatPtr());
      glColor3f(1.0, 1.0, 1.0);
      glBegin(GL_LINES);
        glVertex2f(- 0.1, 0.1); 
        glVertex2f(0.1, -0.1);
        glVertex2f(-0.1, -0.1);
        glVertex2f(0.1, 0.1);
      glEnd();
    glPopMatrix();
  }
  else if (i == "robot1" || i == "robot2" || i == "goal") {
    // we are in robot mode.
    glPushMatrix();
    glLoadIdentity();
    glMultMatrixf(wandMatrix->getFloatPtr());
      // draw a coordinate system.
      glDisable(GL_LIGHTING);
      glBegin(GL_LINES);
        glColor3f(1.0,0.0,0.0);
	glVertex3f(0.0, 0.0, 0.0);
        glVertex3f(1.0, 0.0, 0.0);
	glColor3f(0.0,1.0,0.0);
	glVertex3f(0.0, 0.0, 0.0);
	glVertex3f(0.0, 1.0, 0.0);
	glColor3f(0.0,0.0,1.0);
	glVertex3f(0.0, 0.0, 0.0);
	glVertex3f(0.0, 0.0, 1.0);
      glEnd();
      glEnable(GL_LIGHTING);
      if (i=="robot1") theRollingRobot->draw(1);
      else if (i=="robot2") theFlyingRobot->draw(1);
      else theGoal->draw(1);
    glPopMatrix();  
  }
  else if (i == "idle") {
    // make some fancy triangle
    glPushMatrix(); 
      glLoadIdentity();
      glMultMatrixf(wandMatrix->getFloatPtr());
       glDisable(GL_LIGHTING);
      glBegin(GL_TRIANGLES);
        glColor3f(0.0, 0.5, 1.0);glVertex3f(-0.5, 0.0, 0.0);
	glColor3f(1.0, 0.5, 0.0);glVertex3f(0.5, 0.0, 0.0);
	glColor3f(1.0, 0.5, 1.0);glVertex3f(0.0, 0.5, 0.2);
      glEnd();
      glEnable(GL_LIGHTING);
    glPopMatrix();
  }
  else {
    // pointer type graphic for menu selection.
    glPushMatrix();
      glLoadIdentity();
      glMultMatrixf(wandMatrix->getFloatPtr());
      glColor3f(0.0, 0.5, 1.0);
      glRotatef(-120, 1.0, 0.0, 0.0);
      glutSolidCone(0.2, 0.4, 10, 10);
    glPopMatrix();
  }
  
} // end drawWand().



void engine::updateNavigation() 
{
  vjVec3 xyzAngles;
  vjVec3 xyzTrans;

  vjMatrix    transform, transformIdent;
  vjQuat      source_rot, goal_rot, slerp_rot;
  
  transformIdent.makeIdent();
  source_rot.makeQuat(transformIdent);
   
  vjMatrix* wand_matrix;
  wand_matrix = mWand->getData();
  wand_matrix->getXYZEuler(xyzAngles[0], xyzAngles[1], xyzAngles[2]);
  
  goal_rot.makeQuat(*wand_matrix);
  if(transformIdent != *wand_matrix) {
    slerp_rot.slerp(-0.02, source_rot, goal_rot);
    slerp_rot[0] = 0.0f;
    slerp_rot[2] = 0.0f;
    transform.makeQuaternion(slerp_rot);         
  }
  else
    transform.makeIdent();


  const float velocity_inc = 0.01f;
  if(mButton0->getData() && str == "idle")
    velocity += velocity_inc;
  else if(mButton2->getData())
    velocity = 0.0f;

  vjMatrix local_xform;
  
  local_xform.makeTrans(0, 0, velocity);
  local_xform.postMult(transform);
  
  mNavMatrix.preMult(local_xform);

}




void engine::initGLState()
{
   GLfloat light0_ambient[] = { 0.1f,  0.1f,  0.1f,  1.0f};
   GLfloat light0_diffuse[] = { 0.8f,  0.8f,  0.8f,  1.0f};
   GLfloat light0_specular[] = { 1.0f,  1.0f,  1.0f,  1.0f};
   GLfloat light0_position[] = {0.0f, 10.0f, 10.0f, 0.0f};

   GLfloat mat_ambient[] = { 0.7f, 0.7f,  0.7f,  1.0f};
   GLfloat mat_diffuse[] = { 1.0f,  0.5f,  0.8f,  1.0f};
   GLfloat mat_specular[] = { 1.0f,  1.0f,  1.0f,  1.0f};
   GLfloat mat_shininess[] = { 50.0f};
   GLfloat no_mat[] = { 0.0f,  0.0f,  0.0f,  1.0f};

   glLightfv(GL_LIGHT0, GL_AMBIENT,  light0_ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE,  light0_diffuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR,  light0_specular);
   glLightfv(GL_LIGHT0, GL_POSITION,  light0_position);

   glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient );
   glMaterialfv( GL_FRONT,  GL_DIFFUSE, mat_diffuse );
   glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
   glMaterialfv( GL_FRONT,  GL_SHININESS, mat_shininess );
   glMaterialfv( GL_FRONT,  GL_EMISSION, no_mat);

   glEnable(GL_DEPTH_TEST);
   glEnable(GL_NORMALIZE);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_COLOR_MATERIAL);
   glShadeModel(GL_SMOOTH);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

   glMatrixMode(GL_MODELVIEW);

   long sizeX, sizeY;
   unsigned long *image;
 
   glGenTextures(1, grid_texture);
   glBindTexture(GL_TEXTURE_2D, grid_texture[0]);
   glEnable(GL_TEXTURE_2D);
   image = readLongImageData("./textures/gravel.rgb", &sizeX, &sizeY);
   glTexImage2D(GL_TEXTURE_2D, 0, 4, sizeX, sizeY, 0, 
                 GL_RGBA, GL_UNSIGNED_BYTE, image);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,  GL_MODULATE);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glDisable(GL_TEXTURE_2D); 


   grid_list = glGenLists(1);   
   glNewList(grid_list, GL_COMPILE);
   glBindTexture(GL_TEXTURE_2D, grid_texture[0]);
   glEnable(GL_TEXTURE_2D);
   glScalef(2.0, 2.0, 2.0);
   glColor4f(1.0, 1.0, 1.0, 0.0);
   float step = 2.0;
   for (float j= -10.0; j < 10.0; j = j + step)
     {
       for (float k= 10; k > -10.0; k = k - step)
	 { 
	   glBegin(GL_POLYGON);
	     glTexCoord2i(0,0);glNormal3i(0,1,0);glVertex3d(j, 0, k);
	     glTexCoord2i(0,1);glNormal3i(0,1,0);glVertex3d(j, 0, k-step);
	     glTexCoord2i(1,1);glNormal3i(0,1,0);glVertex3d(j+step, 0, k-step);
	     glTexCoord2i(1,0);glNormal3i(0,1,0);glVertex3d(j+step, 0, k);
	   glEnd();
	 }
     }
   glDisable(GL_TEXTURE_2D);
   glEndList();


   // setting vars and stuff..
   selected_triangle= INF;   
   menuMode= 0;
   str = "idle";
   planner = "";

   show_nodes = false;
   show_path = true;
   animate_path = false;

   myMazeDrawer = new mazeDrawer();

   // setting up the menu stuff
   vjVec3 vec(-0.5, -0.5, -1.0);
   myMenu = new vmenu(2.5, 2.25, vec);

   menuItem *n;
   menuItem *m;
   vector v(3);
   
   v = vector(0.25, 1, -1.0);
   n = new menuItem(0.5, 0.5,v, "./textures/end_draw.rgb", "end_draw");
   myMenu->addMenuItem(n);

   v = vector(0.25, 0.25, -1.0);
   n = new menuItem(0.5, 0.5,v, "./textures/maze_options.rgb", "draw_options");
   myMenu->addMenuItem(n);   
   v= vector(0.5, 0.75, -0.9);
   m= new menuItem(0.75, 0.75, v, "./textures/tofile.rgb", "maze_to_file");
   n->addSubMenu(m);
   v= vector(1.5, 0.75, -0.9);
   m= new menuItem(0.75, 0.75, v, "./textures/fromfile.rgb", "maze_from_file");
   n->addSubMenu(m);
   
   v = vector(0.25, 1.75, -1.0);
   n = new menuItem(0.5, 0.5, v, "./textures/draw.rgb", "draw");
   myMenu->addMenuItem(n); 

   v = vector(1, 0.25, -1.0);
   n = new menuItem(0.5, 0.5, v, "./textures/robot1.rgb", "robot1");
   myMenu->addMenuItem(n);

   v = vector(1, 1,-1.0);
   n = new menuItem(0.5, 0.5,v, "./textures/robot2.rgb", "robot2");
   myMenu->addMenuItem(n);

   v = vector(1, 1.75,-1.0);
   n = new menuItem(0.5, 0.5,v, "./textures/goal.rgb", "goal");
   myMenu->addMenuItem(n); 

   v = vector(1.75, 1,-1.0);
   n = new menuItem(0.5, 0.5,v, "./textures/start.rgb", "start");
   myMenu->addMenuItem(n); 

   v = vector(1.75, 1.75,-1.0);
   n = new menuItem(0.5, 0.5, v, "./textures/stop.rgb", "stop");
   myMenu->addMenuItem(n); 
   
   v = vector(1.75, 0.25,-1.0);
   n = new menuItem(0.5, 0.5,v, "./textures/planning_options.rgb", "run_options");
   myMenu->addMenuItem(n); 
   v= vector(0.25, 0.25,-0.9);
   m= new menuItem(0.5, 0.5,v, "./textures/rrt.rgb", "rrt");
   n->addSubMenu(m);
   v= vector(0.25, 1,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/rrtgoalbias.rgb", "rrtgoalbias");
   n->addSubMenu(m);
   v= vector(0.25, 1.75,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/rrtextcon.rgb", "rrtextcon");
   n->addSubMenu(m);
   v= vector(1, 0.25,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/rrtcon.rgb", "rrtcon");
   n->addSubMenu(m);
   v= vector(1, 1,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/rrtconcon.rgb", "rrtconcon");
   n->addSubMenu(m);
   v= vector(1, 1.75,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/rrtextext.rgb", "rrtextext");
   n->addSubMenu(m);
   v= vector(1.75, 0.25,-0.9);
   m= new menuItem(0.5, 0.5,v, "./textures/shownodes.rgb", "showallnodes");
   n->addSubMenu(m);
   v= vector(1.75, 1,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/showpath.rgb", "showpath");
   n->addSubMenu(m);
   v= vector(1.75, 1.75,-0.9);
   m= new menuItem(0.5, 0.5, v, "./textures/animatepath.rgb", "animatepath");
   n->addSubMenu(m);

   // the robots
   theGoal = new goal();
   theRollingRobot = new rolling();
   theFlyingRobot = new flying();
}














