#ifndef _ENGINEEEE_H_ 
#define _ENGINEEEE_H_ 

/*
 engine.h

 Frode Aarstad, 11.2.2000
 */

#include <vjConfig.h>
#include <iostream.h>
#include <iomanip.h>
#include <Kernel/GL/vjGlApp.h>
#include <Math/vjMatrix.h>
#include <Input/InputManager/vjPosInterface.h>
#include <Input/InputManager/vjDigitalInterface.h>
#include <vjConfig.h>
#include <GL/gl.h>
#include <Math/vjVec3.h>
#include <Math/vjMatrix.h>
#include <Math/vjQuat.h>
#include "load.image.h"

#include "mazeDrawer.h"
#include "vmenu.h"
#include "maze3D.h"
#include "robot.h"
#include "util.h"

#include "world.h"
#include "model.h"
#include "problem.h"
#include "rrt.h"
#include "simulator.h"



class engine : public vjGlApp
{
public:
   engine(vjKernel* kern)
      : vjGlApp(kern)         // Initialize base class
   {}



   ~engine() {
     delete theRollingRobot;
     delete theFlyingRobot;
     delete theGoal;
     delete myMaze3D;
     delete myMazeDrawer;
     delete myMenu;
     delete rrtPlanner1;
     delete rrtPlanner2;
     delete sim1;
     delete sim2;
   }


   
   virtual void init()
   {
      // Initialize devices
      mWand.init("VJWand");
      mHead.init("VJHead");

      mButton0.init("VJButton0");
      mButton1.init("VJButton1");
      mButton2.init("VJButton2");

      mNavMatrix.makeIdent();
   }


   // Execute any initialization needed after API is started
   //  but before the drawManager starts the drawing loops.
   virtual void apiInit()
   {}

   //: Function that is called immediately after a new OGL context is created
   // Initialize GL state here. Also used to create context specific information
   virtual void contextInit()
   {
      initGLState();       // Initialize the GL state information. (lights, shading, etc)
   }

   //: Function to draw the scene
   //! PRE: OpenGL state has correct transformation and buffer selected
   //! POST: The current scene has been drawn
   virtual void draw()
   {
     glClearColor(0.0, 0.0, 0.0, 0.0);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     
     glPushMatrix();
      myDraw();
     glPopMatrix();  
   }

   // ----- Drawing Loop Functions ------
   //
   //  The drawing loop will look similar to this:
   //
   //  while (drawing)
   //  {
   //        preFrame();
   //       draw();
   //        intraFrame();      // Drawing is happening while here
   //       sync();
   //        postFrame();      // Drawing is now done
   //
   //       UpdateTrackers();
   //  }
   //------------------------------------

   //: Function called before updating trackers but after the frame is drawn
   virtual void postFrame()
   {
     if (str == "draw") {
       if (menuMode == 0) {
	 if (button0  == vjDigital::TOGGLE_ON)
	   myMazeDrawer->addPoint(wandMatrix); 
	 else if (button2 == vjDigital::TOGGLE_ON)
	   myMazeDrawer->endShape();
       }
     } 
     
     else if (str == "end_draw" && menuMode == 0) {
       if (myMaze3D == 0x0) 
	 if (myMazeDrawer->getShapeList().size() == 0)
	   myMaze3D = new maze3D("maze.txt");
	 else
	   myMaze3D = new maze3D(myMazeDrawer->getShapeList());
       else
	 myMaze3D->updateMaze(myMazeDrawer->getShapeList());
       str = "idle";
     }

     else if (str == "robot1" && menuMode == 0) {
       theRollingRobot->moveRobot(wandMatrix);
       
       if (button2  == vjDigital::TOGGLE_ON) {
	 theRollingRobot->placeRobot(wandMatrix);
	 str = "idle";
       }
     }
     
     else if (str == "robot2" && menuMode == 0) {
       theFlyingRobot->moveRobot(wandMatrix);
       
       if (button2  == vjDigital::TOGGLE_ON) {
	 theFlyingRobot->placeRobot(wandMatrix);
	 str = "idle";
       }
     }

     else if (str == "goal" && menuMode == 0) {
       theGoal->moveRobot(wandMatrix);
       
       if (button2  == vjDigital::TOGGLE_ON) {
	 theGoal->placeRobot(wandMatrix);
	 str = "idle";
       }
     }
  
     else if (str == "start" && menuMode == 0) {
       if (myMaze3D == 0x0) 
	 cerr << "ERROR: Make a 3d maze first!" <<endl;
       else if (rrtPlanner2 == 0x0) { 
	 rrtPlanner2 = new RRTGoalBias(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
						     new model3DRigid("3D"),
						     theFlyingRobot, theGoal));
       }
       else {
	 Path2.clear();
	 rrtPlanner2->Plan(.25, Path2);
	 sim2 = new simulator(rrtPlanner2);
       }
       str = "idle";
     }

     else if (str == "stop" && menuMode == 0) {
       str = "idle";
     }

     else if (str == "edit_triangle" && menuMode == 0) {
       myMaze3D->selectedVertex(selected_triangle, wandMatrix);
       if (button2 == vjDigital::ON) {
	 myMaze3D->resizeTriangle(selected_triangle, 
				  myMaze3D->selectedVertex(selected_triangle, 
							   wandMatrix),
				  wandMatrix);
       }
     }
     
     else if (str == "maze_to_file") {
       if (myMaze3D != 0x0)
	 myMaze3D->mazeToFile("maze.txt");
       str = "idle";
     }
     
     else if (str == "maze_from_file") {
       if (myMaze3D == 0x0) {
	 myMaze3D = new maze3D("maze.txt");
       }
       else {
	 delete myMaze3D;
	 myMaze3D = new maze3D("maze.txt");
       }
       str = "idle";
     }
     
     else if (str == "rrt") {
       rrtPlanner2 = new RRT(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
					   new model3DRigid("3D"),
					   theFlyingRobot, theGoal));
       str = "idle";
     }
     else if (str == "rrtgoalbias") {
       rrtPlanner2 = new RRTGoalBias(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
						   new model3DRigid("3D"),
						   theFlyingRobot, theGoal));
       str="idle";
     }
     else if (str == "rrtcon") { 
       rrtPlanner2 = new RRTCon(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
					      new model3DRigid("3D"),
					      theFlyingRobot, theGoal));
       str="idle";
     }
     else if (str == "rrtconcon") {
       rrtPlanner2 = new RRTConCon(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
						 new model3DRigid("3D"),
						 theFlyingRobot, theGoal));
       str="idle";
     }
     else if (str == "rrtextcon") { 
       rrtPlanner2 = new RRTExtCon(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
						 new model3DRigid("3D"),
						 theFlyingRobot, theGoal));
       str="idle";
     }
     else if (str == "rrtextext") {
       rrtPlanner2 = new RRTExtExt(new problem3D(new worldPQP(myMaze3D, theFlyingRobot, "empty"),
						 new model3DRigid("3D"),
						 theFlyingRobot, theGoal));
       str="idle";
     }
     else if (str == "showallnodes") {
       show_nodes = !show_nodes;
       str="idle";
     }
     else if (str == "animatepath") {
       animate_path = !animate_path;
       str="idle";
     }
     else if (str == "showpath") {
       show_path = !show_path;
       str="idle";
     }
     
     //---------------------- DRAW PHASE -----------------------------
				
     if (myMaze3D != 0x0) {
       if (str == "idle" && menuMode == 0 && velocity == 0.0f)
	 selected_triangle = myMaze3D->selectedTriangle(wandMatrix);
    
       if (selected_triangle != 99) {
	 if (button0 == vjDigital::TOGGLE_OFF) {
	   velocity = 0.0f;
	   str = "edit_triangle";
	 }
	 else if (button2 == vjDigital::TOGGLE_OFF) str = "idle";
       }
     }
     
     if (menuMode == 1 && button2 == vjDigital::TOGGLE_OFF) {
       str = myMenu->selected(wandMatrix, headMatrix);
       if (myMenu->visibleSubmenu() == 1) myMenu->hideSubmenu();
       else {
	 if (myMenu->drawSubmenu(str, headMatrix) == 0) {
	   menuMode = 0;
	   myMenu->reset();
	 }
       }
     }
     
   } // end postFrame()


   //: Function called after tracker update but before start of drawing
   virtual void preFrame()
   {
     updateNavigation();
     
     // get data from the scene i.e wand and head position
     wandMatrix = mWand->getData();      // Get the wand matrix
     headMatrix = mHead->getData();      // Get the head matrix

     button0 = mButton0->getData();  // get button data. 
     button1 = mButton1->getData(); 
     button2 = mButton2->getData(); 

     if (button1 == vjDigital::TOGGLE_ON && menuMode != 1) menuMode = 1;
     else if (button1 == vjDigital::TOGGLE_ON && menuMode == 1) menuMode = 0;
   }

   //: Function called after drawing has been triggered but BEFORE it completes
   virtual void intraFrame()
   {}


private:
   void myDraw();
   void initGLState();
   void updateNavigation();
   void drawWand(vjMatrix*, string);



public:
   vjPosInterface    mWand;      // the Wand
   vjPosInterface    mHead;      // the head
				      
   vjDigitalInterface mButton0;   
   vjDigitalInterface mButton1;   
   vjDigitalInterface mButton2;   

   int menuMode;
   int grid_list;
   int selected_triangle;
   string str;
   GLuint grid_texture[1];
   string planner;
   
   bool show_nodes;
   bool animate_path;
   bool show_path;
   
   mazeDrawer *myMazeDrawer;
   vmenu      *myMenu;
   maze3D     *myMaze3D;
   goal       *theGoal;
   rolling    *theRollingRobot;
   flying     *theFlyingRobot;

   // path planning
   RRT        *rrtPlanner1;      //simple path planner for rolling robot.
   RRT        *rrtPlanner2;      //path planner for the flying robot.
   list<node> Path1;		 //stores a solution path for rolling robot
   list<node> Path2;	         //stores a solution path for flying robot	
   simulator  *sim1;    
   simulator  *sim2;       

   // navigation.
   vjMatrix mNavMatrix;
   float velocity;
   vjMatrix* wandMatrix;
   vjMatrix* headMatrix;
   int button0;
   int button1;
   int button2;
};
#endif

