/*
  maze3D.cpp

  Frode Aarstad 20.02.2000 edited
 */

#include "maze3D.h"


maze3D::maze3D(list <list <point> > L)
{
  O.clear();
  makeMaze(L);
  
  init();
}



maze3D::maze3D(string file)
{
  // alternative constructor that 
  // reads a maze from a file.

  O.clear();
  mazeFromFile(file);

  init();
}



void maze3D::init() 
{
  //make a PQPmodel of the wand
  PQP_REAL p1[3],p2[3],p3[3];
  p1[0]= 0.0; p1[1]= 0.0; p1[2]= 0.0;
  p2[0]= 0.2; p2[1]= 0.0; p2[2]= 0.0;
  p3[0]= 0.2; p3[1]= 0.2; p3[2]= 0.0;

  wandModel = new PQP_Model();
  wandModel->BeginModel();
  wandModel->AddTri(p1, p2, p3, 0);
  wandModel->EndModel();

  Rwand[0][0] = Rwand[1][1] = Rwand[2][2] = 1.0;
  Rwand[0][1] = Rwand[1][0] = Rwand[2][0] = 0.0;
  Rwand[0][2] = Rwand[1][2] = Rwand[2][1] = 0.0;
  
  Twand[0] = 0.0; Twand[1] = 0.0; Twand[2] = 0.0;

  Robs[0][0] = Robs[1][1] = Robs[2][2] = 1.0;
  Robs[0][1] = Robs[1][0] = Robs[2][0] = 0.0;
  Robs[0][2] = Robs[1][2] = Robs[2][1] = 0.0;
  
  Tobs[0] = 0.0;  Tobs[1] = 0.0; Tobs[2] = 0.0;
}




maze3D::~maze3D()
{
  delete wandModel;
  Obs.clear();
  O.clear();
  model.clear();
}



void maze3D::mazeFromFile(string file)
{
  // not totally ok.

  list <point> obst;
  list <list <point> > all_obst;
  
  file_istream fin(file);
  while (!fin.eof()) {
    fin >> obst;
    all_obst.push_back(obst);
  }

  //  cout << "in :"<<all_obst<<", size: "<<all_obst.size()<<endl;

  makeMaze(all_obst);
  all_obst.clear();
  fin.close();
}



void maze3D::mazeToFile(string file)
{
  file_ostream fout(file);
  fout << model;
  fout.close();
}



void maze3D::makeMaze(list <list <point> > L) {
  // make a displayable version of the maze
    
  list <point> M;
  point pt1;
  point pt2;
  triangle3d *tri1;
  triangle3d *tri2;

  forall (M, L) {
    // get an obstacle M
    for (int i= 0; i < M.size()-1; i++) {
      tri1 = new triangle3d();
      tri2 = new triangle3d();
      pt1 = M.inf(M[i]);
      pt2 = M.inf(M[i+1]);

      // translate from drawingboard to world coordinates...
      float pt1x = ((float)pt1.xcoord() - 5) *4;
      float pt1y = ((float)pt1.ycoord() - 5) *-4;
      float pt2x = ((float)pt2.xcoord() - 5) *4;
      float pt2y = ((float)pt2.ycoord() - 5) *-4;

      // the first triangle.
      tri1->p0[0] = pt1x;tri1->p0[1] =  0.0;tri1->p0[2] = pt1y;
      tri1->p1[0] = pt1x;tri1->p1[1] = 10.0;tri1->p1[2] = pt1y;
      tri1->p2[0] = pt2x;tri1->p2[1] = 10.0;tri1->p2[2] = pt2y;

      // the second triangle.
      tri2->p0[0] = pt1x;tri2->p0[1] =  0.0;tri2->p0[2] = pt1y;
      tri2->p1[0] = pt2x;tri2->p1[1] =  0.0;tri2->p1[2] = pt2y;
      tri2->p2[0] = pt2x;tri2->p2[1] = 10.0;tri2->p2[2] = pt2y;

      // get the normal.
      double a[3],b[3];
      VmV(a,tri1->p1,tri1->p0);
      VmV(b,tri1->p2,tri1->p0);
      VcrossV(tri1->n,a,b);
      Vnormalize(tri1->n);
      
      VmV(a,tri2->p1,tri2->p0);
      VmV(b,tri2->p2,tri2->p0);
      VcrossV(tri2->n,a,b);
      Vnormalize(tri2->n);

      O.push_back(tri1);
      O.push_back(tri2);
    }
  }

  makePQPModel(INF);
  model= L;
  obstSize = L.size();

} //end makeMaze().



void maze3D::makePQPModel(int nr) {
  // make the PQP_model
  
  PQP_Model *p;
  triangle3d *nt;

  if (nr == INF) { 
    Obs.clear();
    for (int j= 0; j < O.size(); j++) {
      p = new PQP_Model();
      p->BeginModel();
      nt = O.inf(O[j]);
      p->AddTri(nt->p0, nt->p1, nt->p2, j);
      p->EndModel();
      Obs.push_back(p);
    }
  }
  else {
    p = new PQP_Model();
    p->BeginModel();
    nt = O.inf(O[nr]);
    p->AddTri(nt->p0, nt->p1, nt->p2, nr);
    p->EndModel();
    Obs.assign(Obs[nr], p);
  }

} //end makePQPModel().



void maze3D::draw(int triangle_nr)
{
  triangle3d *t;

  glPushMatrix();
  glColor3f(0.1, 0.9, 0.1);
  glBegin(GL_TRIANGLES);
  for (int i= 0; i < O.size(); i++) {
    t = O.inf(O[i]);
    if (i == triangle_nr) glColor3f(0.1, 0.1, 0.7);
    else glColor3f(0.1, 0.9, 0.1);
    glNormal3f(t->n[0],  t->n[1],  t->n[2]);
    glVertex3f(t->p0[0], t->p0[1], t->p0[2]);
    glVertex3f(t->p1[0], t->p1[1], t->p1[2]);
    glVertex3f(t->p2[0], t->p2[1], t->p2[2]);
  }
  glEnd();
  glPopMatrix();

  if (triangle_nr != 99) {
    // placing points at the vertex of the selected triangle.
    glColor3f(1.0, 0.0, 0.0); 
    
    t = O.inf(O[triangle_nr]);
    glPushMatrix();
    glTranslatef(t->p0[0], t->p0[1], t->p0[2]);
    glutSolidSphere(.15, 10, 10);
    glPopMatrix();
    
    glColor3f(0.0, 1.0, 0.0);
    glPushMatrix();
    glTranslatef(t->p1[0], t->p1[1], t->p1[2]);
    glutSolidSphere(.15, 10, 10);
    glPopMatrix();
    
    glColor3f(0.0, 0.0, 1.0);
    glPushMatrix();
    glTranslatef(t->p2[0], t->p2[1], t->p2[2]);
    glutSolidSphere(.15, 10, 10);
    glPopMatrix();
  }
  
}  //end draw().



int maze3D::selectedTriangle(vjMatrix *wand) {
  // picks a corner of triangle and returns
  // the triangle number

  int tmp = 99;
  float x,y,z;

  PQP_DistanceResult dres;
  PQP_REAL P1[3],V1[3];
  PQP_REAL P2[3],V2[3];

  wand->getTrans(x,y,z);
  Twand[0] = (PQP_REAL)x;  Twand[1] = (PQP_REAL)y; Twand[2] = (PQP_REAL)z;
  
  for (int i=0; i<Obs.size(); i++) {
    PQP_Distance(&dres,Robs,Tobs, Obs.inf(Obs[i]), 
		 Rwand, Twand, wandModel, 0.0,0.0);
    
    VcV(P1,dres.P1());
    VcV(P2,dres.P2());
    MxVpV(V1,Robs,P1,Tobs);
    MxVpV(V2,Rwand,P2,Twand);

    if (euclidianDist(V1, V2) < 1.5) tmp = i;
  }  

  return tmp;

} // end selectedTriangle().



bool maze3D::collisionFree(robot *R, double tolerance) {
  // returns true if collision occurs within
  // the tolerance region.

  PQP_ToleranceResult tres;  // this is faster than the PQP_Distance query.
  bool val;
  
  for (int i=0; i<Obs.size(); i++)
    PQP_Tolerance(&tres,Robs,Tobs, Obs.inf(Obs[i]),
		  R->R, R->T, R->getRobotModel(), (PQP_REAL)tolerance);
  
  // need both checks because of the strange PQP_REAL
  if (tres.Distance() == 0 || (float)tres.Distance() != 0.0f) val= false;
  else val= true;

  return val;
}



float maze3D::collisionDistance(robot *R) {
  // returns the distance to collision.

  PQP_DistanceResult dres;
  float min = INF;

  for (int i=0; i<Obs.size(); i++) {
    PQP_Distance(&dres,Robs,Tobs, Obs.inf(Obs[i]),
		 R->R, R->T, R->getRobotModel(), 0.0, 0.0);
    if ((float)dres.Distance() < min) min= (float)dres.Distance();
  }
  return min;
}



int maze3D::selectedVertex(int nr, vjMatrix *M) {

  float x,y,z;
  int tmp = INF;
  triangle3d *t;
  const float radii = 2.5; 

  M->getTrans(x,y,z);
  t= O.inf(O[nr]);
  
  if ((x < t->p0[0]+radii && x > t->p0[0]-radii) &&
      (y < t->p0[1]+radii && y > t->p0[1]-radii) &&
      (z < t->p0[2]+radii && z > t->p0[2]-radii))
    tmp=  0; 
  else if ((x < t->p1[0]+radii && x > t->p1[0]-radii) &&
	   (y < t->p1[1]+radii && y > t->p1[1]-radii) &&
	   (z < t->p1[2]+radii && z > t->p1[2]-radii)) 
    tmp= 1; 
  else if ((x < t->p2[0]+radii && x > t->p2[0]-radii) &&
	   (y < t->p2[1]+radii && y > t->p2[1]-radii) &&
	   (z < t->p2[2]+radii && z > t->p2[2]-radii)) 
    tmp= 2; 
  
  // draw a coordinate system on selected corner.
  glPushMatrix();

  if (tmp == 0) glTranslatef(t->p0[0], t->p0[1], t->p0[2]);
  else if (tmp == 1) glTranslatef(t->p1[0], t->p1[1], t->p1[2]);
  else glTranslatef(t->p2[0], t->p2[1], t->p2[2]);
  
  glDisable(GL_LIGHTING);
  glColor3f(1.0, 0.0, 0.0);
  glBegin(GL_LINES);
   glVertex3f(0.0, 0.0, 1.0);
   glVertex3f(0.0, 0.0, -1.0);
  glEnd();
  glColor3f(0.0, 1.0, 0.0);
  glBegin(GL_LINES);
   glVertex3f(1.0, 0.0, 0.0);
   glVertex3f(-1.0, 0.0, 0.0);
  glEnd();
  glColor3f(0.0, 0.0, 1.0);
  glBegin(GL_LINES);
   glVertex3f(0.0, 1.0, 0.0);
   glVertex3f(0.0, -1.0, 0.0);
  glEnd();
  glEnable(GL_LIGHTING);

  glPopMatrix();

  return tmp;
}



void maze3D::resizeTriangle(int obst, int vertex, vjMatrix *M) {
  
  float x,y,z;
  triangle3d *t;

  M->getTrans(x,y,z);
  t= O.inf(O[obst]);

  // replace the current triangles point with the 
  // uptaded one (x,y,z)

  if (vertex == 0) {
    t->p0[0] = x;
    t->p0[1] = y;
    t->p0[2] = z;
  }
  else if (vertex == 1) {
    t->p1[0] = x;
    t->p1[1] = y;
    t->p1[2] = z;
  }
  else {
    t->p2[0] = x;
    t->p2[1] = y;
    t->p2[2] = z;
  }
  
  O.assign(O[obst], t);

  draw(obst);
  makePQPModel(obst);
}



void maze3D::updateMaze(list <list <point> > L) {
  // updates the maze with some new obstacles.

  list <list <point> > L_new;
  L_new.clear();
  
  for (int i= 0; i < (L.size()-obstSize); i++) 
    L_new.push(L.inf(L[i]));
  
  obstSize = L.size();
  makeMaze(L_new);
  L_new.clear();
}




/*
  SOME MATH STUFF
 */


float maze3D::euclidianDist(PQP_REAL a[3], PQP_REAL b[3]) 
{
 return sqrt(pow((a[0]-b[0]),2) + pow((a[1] - b[1]),2) + pow((a[2] -
b[2]),2));
}


void maze3D::VmV(double Vr[3], const double V1[3], const double V2[3])
{
  Vr[0] = V1[0] - V2[0];
  Vr[1] = V1[1] - V2[1];
  Vr[2] = V1[2] - V2[2];
}


void maze3D::VcrossV(double Vr[3], const double V1[3], const double V2[3])
{
  Vr[0] = V1[1]*V2[2] - V1[2]*V2[1];
  Vr[1] = V1[2]*V2[0] - V1[0]*V2[2];
  Vr[2] = V1[0]*V2[1] - V1[1]*V2[0];
}


void maze3D::Vnormalize(double V[3])
{
  double d = 1.0 / sqrt(V[0]*V[0] + V[1]*V[1] + V[2]*V[2]);
  V[0] *= d;
  V[1] *= d;
  V[2] *= d;
}



// end maze3D.cpp






