#ifndef _MAZED3D_H_
#define _MAZED3D_H_
/*
  maze3D.h

  Frode Aarstad, 20.02.2000 edited
 */

#include <LEDA/list.h>
#include <LEDA/point.h>
#include <LEDA/stream.h>
#include <LEDA/file.h>
#include <GL/gl.h>
#include <Math/vjMatrix.h>
#include "MatVec.h"
#include "util.h"
#include "robot.h"

#define WALL_SIZE 10             // size of the drawing wall


class maze3D {

public:
  maze3D(list <list <point> >);
  maze3D(string);
  ~maze3D();

  void draw(int);
  int selectedTriangle(vjMatrix *);
  int selectedVertex(int, vjMatrix *);
  void resizeTriangle(int, int, vjMatrix *);
  float collisionDistance(robot *);
  bool collisionFree(robot *, double);
  void updateMaze(list <list <point> >);

  void mazeFromFile(string);
  void mazeToFile(string);


private:
  void makePQPModel(int);
  void makeMaze(list <list <point> >);
  void init();

  void Vnormalize(double[]);
  void VcrossV(double[3], const double[3], const double[3]);
  void VmV(double[3], const double[3], const double[3]);
  float euclidianDist(PQP_REAL[3], PQP_REAL[3]);


  PQP_Model *wandModel;
  PQP_REAL Rwand[3][3], Twand[3];
  PQP_REAL Robs[3][3] , Tobs[3];

  list <PQP_Model *> Obs;      
  list <triangle3d *> O;      
  list <list <point> > model;

  int obstSize;
};
#endif


































