/*
  mazeDrawer.cpp

  Frode Aarstad, 14.02.2000
 */

#include "mazeDrawer.h"


mazeDrawer::mazeDrawer()
{
  glEnable(GL_LINE_SMOOTH);
  aShape.clear();
  allShapes.clear();
}


mazeDrawer::~mazeDrawer()
{
  aShape.clear();
  allShapes.clear();
}


void mazeDrawer::endShape()
{
  // ending the wall. and starting a new one.

  allShapes.push(aShape);
  aShape.clear();
  draw();
}



void mazeDrawer::addPoint(vjMatrix *wand)
{
  // adding points to the wall.

  float x;
  float y;
  float z;
  wand->getTrans(x,y,z);

  if (x > WALL_SIZE) x = (float)WALL_SIZE;
  else if (x < 0) x = 0;
  if (y > WALL_SIZE) y = (float)WALL_SIZE;
  else if (y < 0) y = 0;

  point pt((double)x, (double)y);
  aShape.push(pt);
}



void mazeDrawer::draw()
{
  // drawing the walls of the maze.

  point pt;
  list <point> L;

  glPushMatrix();
  glLineWidth(2.0);
  glColor3f(0.1, 0.1, 1.0);
  glBegin(GL_LINE_STRIP);
  forall(pt, aShape) {
    glVertex2f((float)pt.xcoord(), (float)pt.ycoord()); 
  }
  glEnd();
  glLineWidth(1.0);
  glPopMatrix();
  
  if (allShapes.size() != 0) {
    // draw stuff already stored
    glPushMatrix();
    glLineWidth(2.0);
    glColor3f(0.1, 0.5, 0.8);
    forall(L, allShapes) {
      glBegin(GL_LINE_STRIP);
      forall(pt, L) {
	glVertex2f((float)pt.xcoord(), (float)pt.ycoord()); 
      }
      glEnd();
    }
    glLineWidth(1.0);
    glPopMatrix();
  }

} // end draw().



void mazeDrawer::addWall()
{
  // drawing the wall to paint upon

  glPushMatrix();
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glColor4f(0.8, 0.2, 0.0, 0.0);
  for (float j= 0.0; j < WALL_SIZE; j = j + 0.25)
  {
    for (float k= WALL_SIZE; k > 0.0; k = k - 0.25)
    { 
      glBegin(GL_POLYGON);
        glVertex3d(j, k, 0);
        glVertex3d(j, k-0.25, 0);
        glVertex3d(j+0.25, k-0.25, 0);
        glVertex3d(j+0.25, k, 0); 
      glEnd();
    }
  }
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glPopMatrix();
}



list <list <point> > mazeDrawer::getShapeList()
{
  return allShapes;
}


// end class mazeDrawer().








