/*
Com S 476/576 Course Software 
Release 0.1  02/22/00

Copyright 1999,2000

Steven M. LaValle, Iowa State University
*/
#include <fstream.h>
#include <math.h>

#include <LEDA/file.h>
#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/random.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>


#include "world.h"
#include "model.h"
#include "problem.h"


// Constructor
Problem::Problem(World *world, Model *model, robot *r, robot *g, string path = "") {

  SetWorld(world);
  SetModel(model);

  theRobot = r;
  theGoal = g;

  FilePath = path;

  if (is_file(FilePath+"InitialState"))
    ReadInitialState();
  else 
    InitialState = vector((double)theRobot->pos[0],
                          (double)theRobot->pos[2]);
  
  if (is_file(FilePath+"GoalState"))
    ReadGoalState();
  else
    GoalState = vector((double)theGoal->pos[0],
                       (double)theGoal->pos[2]);



  StateDim = M->StateDim;
  InputDim = M->InputDim;
  LowerState = M->LowerState;
  UpperState = M->UpperState;
}


void Problem::SetWorld(World *world) {
  W = world;
}


void Problem::SetModel(Model *model) {
  M = model;
  StateDim = M->StateDim;
  InputDim = M->InputDim;
  LowerState = M->LowerState;
  UpperState = M->UpperState;
}


void Problem::ReadLowerState() {
  file_istream fin(FilePath + "LowerState");
  fin >> LowerState;
}


void Problem::ReadUpperState() {
  file_istream fin(FilePath + "UpperState");
  fin >> UpperState;
}


void Problem::ReadInitialState() {
  file_istream fin(FilePath + "InitialState");
  fin >> InitialState;
}


void Problem::ReadGoalState() {
  file_istream fin(FilePath + "GoalState");
  fin >> GoalState;
}


// In the base class, steal the following methods from Model

list<vector> Problem::GetInputs(const vector &x) {
  return M->GetInputs(x);
}


vector Problem::LinearInterpolate(const vector &x1, const vector &x2, 
				  const double &a) {
  return M->LinearInterpolate(x1,x2,a);
}



point Problem::StateToPoint(const vector &x) {
  return M->StateToPoint(x);
}

// By default, don't change anything
vector Problem::StateToConfiguration(const vector &x) {
  return M->StateToConfiguration(x);
}


// Default metric: use the metric from the model
double Problem::Metric(const vector &x1, const vector &x2) {
  return M->Metric(x1,x2);
}


bool Problem::Satisfied(const vector &x) {
  return ((W->CollisionFree(StateToConfiguration(x)))&&
	  (M->Satisfied(x)));
}

vector Problem::Integrate(const vector &x, const vector &u, const double &h) {
  return M->Integrate(x,u,h);
} 


// In the base class, steal the following methods from World
bool Problem::CollisionFree(const vector &q) {
  return W->CollisionFree(q);
}


double Problem::DistanceComp(const vector &q) {
  return W->DistanceComp(q);
}


list<polygon> Problem::EnvironmentToPolygons() {
  return W->EnvironmentToPolygons();
}


list<polygon> Problem::RobotToPolygons(const vector &q) {
  return W->RobotToPolygons(q);
}



////////////////////////////////////////////////////////////
//
// problem3D
// Frode Aarstad
//
////////////////////////////////////////////////////////////

problem3D::problem3D(World *world, Model *model, 
		     robot *r, robot *g)
  :Problem(world, model, r, g, "") 
{
  StateDim = M->StateDim;
  InputDim = M->InputDim;
  LowerState = M->LowerState;
  UpperState = M->UpperState;

  InitialState = vector(StateDim);
  GoalState = vector(StateDim);

  InitialState[0] = (double)theRobot->pos[0];
  InitialState[1] = (double)theRobot->pos[1];
  InitialState[2] = (double)theRobot->pos[2];
  InitialState[3] = PI;

  GoalState[0] = (double)theGoal->pos[0];
  GoalState[1] = (double)theGoal->pos[1]+1.0;
  GoalState[2] = (double)theGoal->pos[2];
  GoalState[3]= PI;
}

  








