#ifndef _PROBLEM_H_ 
#define _PROBLEM_H_
/*
Com S 476/576 Course Software 
Release 0.1  02/22/00

Copyright 1999,2000

Steven M. LaValle, Iowa State University
*/

#include "robot.h"

class Problem {
 protected:
  World *W;  // Need to define a world
  Model *M;  // xdot = f(x,u), integration technique, state bounds
  robot *theRobot;  //the robot
  robot *theGoal;
 public:
  string FilePath;     // This filepath is used for a particular problem
  int StateDim;        // The dimension of the state space
  int InputDim;        // The dimension of the input vector
  vector LowerState;   // Vector of minimum values for each state variable
  vector UpperState;   // Vector of maximum values for each state variable
  vector InitialState; 
  vector GoalState;

  // The constructor accepts a world, a model, and a path
  Problem(World *world, Model *model, robot *, robot *, string path);
  virtual ~Problem() {};

  // Some useful methods for changing the world or model (often in a GUI)
  void SetWorld(World *world);  
  void SetModel(Model *model); 

  // Some methods to read from a file
  void ReadLowerState();
  void ReadUpperState();
  void ReadInitialState();
  void ReadGoalState();

  // The following methods pass arguments to equivalent methods in Model
  virtual list<vector> GetInputs(const vector &x);
  virtual vector Integrate(const vector &x, const vector &u, const double &h); 
  virtual vector LinearInterpolate(const vector &x1, const vector &x2, 
				   const double &a);  // Depends on topology
  virtual double Metric(const vector &x1, const vector &x2);
  virtual vector StateToConfiguration(const vector &x);
  virtual point StateToPoint(const vector &x); // Get a 2D point from state

  // The following methods pass arguments to equivalent methods in World
  virtual bool CollisionFree(const vector &q); // Input is configuration
  virtual double DistanceComp(const vector &q);  // Distance in world
  virtual list<polygon> EnvironmentToPolygons();
  virtual list<polygon> RobotToPolygons(const vector &q);

  // This takes the logical AND of CollisionFree from World, and 
  // Satisfied from Model
  virtual bool Satisfied(const vector &x);

};


////////////////////////////////////////////////////////////
//
// problem3D
//
////////////////////////////////////////////////////////////

class problem3D: public Problem 
{
public:
  problem3D(World *world, Model *model, robot *, robot *);
  

private:

};
#endif

