/*
  robot.cpp

  Frode Aarstad, 20.02.2000
 */
#include "robot.h"


robot::robot(){

  R[0][0] = R[1][1] = R[2][2] = 1.0;
  R[0][1] = R[1][0] = R[2][0] = 0.0;
  R[0][2] = R[1][2] = R[2][1] = 0.0;
 
  T[0] = 0.0;  T[1] = 0.0; T[2] = 0.0;

  placed = 0;
}


robot::~robot()
{
  delete robotModel;
  O.clear();
}



PQP_Model * robot::getRobotModel() {
  return robotModel;
}



void robot::draw(int i) {

  if (i == 1)
    glCallList(robotList);
  if (placed == 1 && i == 0) {
    glPushMatrix();
      glTranslatef(pos[0], pos[1], pos[2]);
      glCallList(robotList);
    glPopMatrix();
  }
}



void robot::moveRobot(vjMatrix *M) 
{
  M->getTrans(pos[0],pos[1],pos[2]);
  T[0] = (PQP_REAL)pos[0];
  T[1] = (PQP_REAL)pos[1];
  T[2] = (PQP_REAL)pos[2];
}


/*
  The robots

 */

goal::goal()
  :robot()
{ 
  list <triangle3d *> model;
  triangle3d *tri;
  tri = new triangle3d();
  
  tri->p0[0] = -0.05;tri->p0[1] = 0.0;tri->p0[2] = 0.0;
  tri->p1[0] = 0.0  ;tri->p1[1] = 2.0;tri->p1[2] = 0.0;
  tri->p2[0] = 0.05 ;tri->p2[1] = 0.0;tri->p2[2] = 0.0;
  tri->n[0] = 0.0; tri->n[1] = 0.0; tri->n[2] = 1.0;
  tri->col[0] = 1.0; tri->col[1] = 1.0;tri->col[2] = 0.0; 
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = 0.0;tri->p0[1] = 1.5;tri->p0[2] = 0.0;
  tri->p1[0] = 0.0;tri->p1[1] = 2.0;tri->p1[2] = 0.0;
  tri->p2[0] = 1.0;tri->p2[1] = 1.5;tri->p2[2] = 0.0;
  tri->n[0] = 0.0; tri->n[1] = 0.0; tri->n[2] = 1.0;
  tri->col[0] = 1.0; tri->col[1] = 1.0;tri->col[2] = 0.0; 
  model.push_back(tri);

  robotList = glGenLists(1);
  glNewList(robotList,GL_COMPILE);
  glBegin(GL_TRIANGLES);
  triangle3d *t;
    for (int i= 0; i < model.size(); i++) {
      t = model.inf(model[i]);
      glColor3f(t->col[0], t->col[1], t->col[2]);
      glNormal3f(t->n[0],  t->n[1],  t->n[2]);
      glVertex3f(t->p0[0], t->p0[1], t->p0[2]);
      glVertex3f(t->p1[0], t->p1[1], t->p1[2]);
      glVertex3f(t->p2[0], t->p2[1], t->p2[2]);
    }
  glEnd();
  glEndList();  

  robotModel = new PQP_Model();
  robotModel->BeginModel();
  triangle3d *nt;
  for (int j= 0; j < model.size(); j++) {
    nt = model.inf(model[j]);
    robotModel->AddTri(nt->p0, nt->p1, nt->p2, j);
  }
  robotModel->EndModel();
}



void goal::placeRobot(vjMatrix *M)
{
  float y;
  M->getTrans(pos[0],y,pos[2]);
  pos[1] = 0.1;
  placed = 1;
}



/*
  Rolling robot
 */

rolling::rolling()
  :robot()
{  
  list <triangle3d *> model;
  triangle3d *tri;
  tri = new triangle3d();

  tri->p0[0] = 0.0;tri->p0[1] = 0.0;tri->p0[2] = 0.2;
  tri->p1[0] = 0.2;tri->p1[1] = 0.2;tri->p1[2] = 0.0;
  tri->p2[0] =-0.2;tri->p2[1] = 0.2;tri->p2[2] = 0.0;
  model.push_back(tri);
  
  robotList = glGenLists(1);
  glNewList(robotList,GL_COMPILE);
  glColor3f(0.3, 0.5, 0.76);
  glutSolidSphere(.4, 10, 10);
  glEndList();  

  // making the PQP model for the robot
  robotModel = new PQP_Model();
  robotModel->BeginModel();
  triangle3d *nt;
  for (int j= 0; j < model.size(); j++) {
    nt = model.inf(model[j]);
    robotModel->AddTri(nt->p0, nt->p1, nt->p2, j);
  }
  robotModel->EndModel();
}


void rolling::placeRobot(vjMatrix *M)
{
  float y;
  M->getTrans(pos[0],y,pos[2]);
  pos[1] = 0.1;
  placed = 1;
}


/*
  Flying robot
 */

flying::flying()
  :robot()
{
  list <triangle3d *> model;
  triangle3d *tri;
  tri = new triangle3d();

  //body
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0;tri->p0[2] = 0.0;
  tri->p1[0] =-0.5 ;tri->p1[1] = -1.0;tri->p1[2] = 0.5;
  tri->p2[0] = 0.5 ;tri->p2[1] = -1.0;tri->p2[2] = 0.5;
  tri->n[0] = 0.0; tri->n[1] = 0.5; tri->n[2] = 0.5;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;  
  model.push_back(tri);
  
  tri = new triangle3d();
  tri->p0[0] =  0.0;tri->p0[1] =  0.0;tri->p0[2] =  0.0;
  tri->p1[0] = -0.5;tri->p1[1] = -1.0;tri->p1[2] = -0.5;
  tri->p2[0] =  0.5;tri->p2[1] = -1.0;tri->p2[2] = -0.5;
  tri->n[0] = 0.0; tri->n[1] = 0.5; tri->n[2] = -0.5;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = -0.5;tri->p0[1] = -1.0;tri->p0[2] = -0.5;
  tri->p1[0] = -0.0;tri->p1[1] = -0.0;tri->p1[2] = -0.0;
  tri->p2[0] = -0.5;tri->p2[1] = -1.0;tri->p2[2] =  0.5;
  tri->n[0] = -0.5; tri->n[1] = 0.5; tri->n[2] = 0.0;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] =  0.5;tri->p0[1] = -1.0;tri->p0[2] = -0.5;
  tri->p1[0] =  0.0;tri->p1[1] = -0.0;tri->p1[2] =  0.0;
  tri->p2[0] =  0.5;tri->p2[1] = -1.0;tri->p2[2] =  0.5;
  tri->n[0] = 0.5; tri->n[1] = 0.5; tri->n[2] = 0.0;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);


  //front
  tri = new triangle3d();
  tri->p0[0] = -1.0 ;tri->p0[1] = -1.0;tri->p0[2] = 0.0;
  tri->p1[0] = -0.5 ;tri->p1[1] = -1.0;tri->p1[2] = 0.5;
  tri->p2[0] = -0.25;tri->p2[1] = -0.5;tri->p2[2] = 0.0;
  tri->n[0] = -0.25; tri->n[1] = 0.25; tri->n[2] = 0.25;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = -1.0 ;tri->p0[1] = -1.0;tri->p0[2] =  0.0;
  tri->p1[0] = -0.5 ;tri->p1[1] = -1.0;tri->p1[2] = -0.5;
  tri->p2[0] = -0.25;tri->p2[1] = -0.5;tri->p2[2] = -0.0;
  tri->n[0] = -0.25; tri->n[1] = 0.25; tri->n[2] = -0.25;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);

  // tail
  tri = new triangle3d();
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0;tri->p0[2] =  0.0;
  tri->p1[0] = 1.5 ;tri->p1[1] = -0.15;tri->p1[2] =  0.0;
  tri->p2[0] = 0.1;tri->p2[1] = -0.40;tri->p2[2] = -0.15;
  tri->n[0] = -0.25; tri->n[1] = 0.25; tri->n[2] = 0.25;
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0;tri->p0[2] =  0.0;
  tri->p1[0] = 1.5 ;tri->p1[1] =-0.15;tri->p1[2] =  0.0;
  tri->p2[0] = 0.1;tri->p2[1] = -0.40;tri->p2[2] =  0.15;
  tri->n[0] = 0.25; tri->n[1] = 0.25; tri->n[2] = 0.25; 
  tri->col[0] = 0.0; tri->col[1] = 0.2;tri->col[2] = 0.8;
  model.push_back(tri);

  // back rotor
  tri = new triangle3d();
  tri->p0[0] = 1.3 ;tri->p0[1] = -0.15;tri->p0[2] =  0.1;
  tri->p1[0] = 1.4 ;tri->p1[1] = -0.3 ;tri->p1[2] =  0.1;
  tri->p2[0] = 1.2 ;tri->p2[1] = -0.3 ;tri->p2[2] =  0.1;
  tri->n[0] = 0.0  ;tri->n[1] = 0.0   ;tri->n[2] = 1.0; 
  tri->col[0] = 0.5; tri->col[1] = 0.5;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = 1.3 ;tri->p0[1] =  -0.15;tri->p0[2] =  0.1;
  tri->p1[0] = 1.4 ;tri->p1[1] = -0.0  ;tri->p1[2] =  0.1;
  tri->p2[0] = 1.2 ;tri->p2[1] = -0.0  ;tri->p2[2] =  0.1;
  tri->n[0] = 0.0  ;tri->n[1] = 0.0    ;tri->n[2] = 1.0; 
  tri->col[0] = 0.5; tri->col[1] = 0.5;tri->col[2] = 0.8;
  model.push_back(tri); 

  // top rotor
  tri = new triangle3d();
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0 ;tri->p0[2] =  0.0;
  tri->p1[0] =-2.0 ;tri->p1[1] =  0.0 ;tri->p1[2] = -0.1;
  tri->p2[0] =-2.0 ;tri->p2[1] =  0.0 ;tri->p2[2] =  0.1;
  tri->n[0] = 0.0  ;tri->n[1] = 1.0   ;tri->n[2] = 1.0; 
  tri->col[0] = 0.5; tri->col[1] = 0.5;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0 ;tri->p0[2] =  0.0;
  tri->p1[0] = 2.0 ;tri->p1[1] =  0.0 ;tri->p1[2] = -0.1;
  tri->p2[0] = 2.0 ;tri->p2[1] =  0.0 ;tri->p2[2] =  0.1;
  tri->n[0] = 0.0  ;tri->n[1] = 1.0   ;tri->n[2] = 1.0; 
  tri->col[0] = 0.5; tri->col[1] = 0.5;tri->col[2] = 0.8;
  model.push_back(tri);

  tri = new triangle3d();
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0 ;tri->p0[2] =  0.0;
  tri->p1[0] = 0.1 ;tri->p1[1] =  0.0 ;tri->p1[2] =  2.0;
  tri->p2[0] =-0.1 ;tri->p2[1] =  0.0 ;tri->p2[2] =  2.0;
  tri->n[0] = 0.0  ;tri->n[1] = 1.0   ;tri->n[2] = 1.0; 
  tri->col[0] = 0.5; tri->col[1] = 0.5;tri->col[2] = 0.8;
  model.push_back(tri);
 
  tri = new triangle3d();
  tri->p0[0] = 0.0 ;tri->p0[1] =  0.0 ;tri->p0[2] =  0.0;
  tri->p1[0] = 0.1 ;tri->p1[1] =  0.0 ;tri->p1[2] = -2.0;
  tri->p2[0] =-0.1 ;tri->p2[1] =  0.0 ;tri->p2[2] = -2.0;
  tri->n[0] = 0.0  ;tri->n[1] = 1.0   ;tri->n[2] = 1.0; 
  tri->col[0] = 0.5; tri->col[1] = 0.5;tri->col[2] = 0.8;
  model.push_back(tri);

  robotList = glGenLists(1);
  glNewList(robotList,GL_COMPILE);
  glBegin(GL_TRIANGLES);
  triangle3d *t;
  for (int i= 0; i < model.size()-6; i++) {
    t = model.inf(model[i]);
    glColor3f(t->col[0], t->col[1], t->col[2]);
    glNormal3f(t->n[0],  t->n[1],  t->n[2]);
    glVertex3f(t->p0[0], t->p0[1], t->p0[2]);
    glVertex3f(t->p1[0], t->p1[1], t->p1[2]);
    glVertex3f(t->p2[0], t->p2[1], t->p2[2]);
  }
  glEnd();
  glEndList();  

  smallRotorList = glGenLists(1);
  glNewList(smallRotorList,GL_COMPILE);
  glBegin(GL_TRIANGLES);
  for (i= model.size()-6; i < model.size()-4; i++) {
    t = model.inf(model[i]);
    glColor3f(t->col[0], t->col[1], t->col[2]);
    glNormal3f(t->n[0],  t->n[1],  t->n[2]);
    glVertex3f(t->p0[0], t->p0[1], t->p0[2]);
    glVertex3f(t->p1[0], t->p1[1], t->p1[2]);
    glVertex3f(t->p2[0], t->p2[1], t->p2[2]);
  }
  glEnd();
  glEndList();  


  bigRotorList = glGenLists(1);
  glNewList(bigRotorList,GL_COMPILE);
  glBegin(GL_TRIANGLES);
  for (i= model.size()-4; i < model.size(); i++) {
    t = model.inf(model[i]);
    glColor3f(t->col[0], t->col[1], t->col[2]);
    glNormal3f(t->n[0],  t->n[1],  t->n[2]);
    glVertex3f(t->p0[0], t->p0[1], t->p0[2]);
    glVertex3f(t->p1[0], t->p1[1], t->p1[2]);
    glVertex3f(t->p2[0], t->p2[1], t->p2[2]);
  }
  glEnd();
  glEndList();  


  // making the PQP model for the robot
  robotModel = new PQP_Model();
  robotModel->BeginModel();
  triangle3d *nt;
  for (int j= 0; j < model.size(); j++) {
    nt = model.inf(model[j]);
    robotModel->AddTri(nt->p0, nt->p1, nt->p2, j);
  }
  robotModel->EndModel();

  bigProp = 0.0f;
}


void flying::placeRobot(vjMatrix *M)
{
  // placing the robot, and making sure that
  // it is above the xz=plane
  
  float y;
  M->getTrans(pos[0],y,pos[2]);
  if (y < 0) y = 0.0f;
  pos[1] = y;
  placed = 1;
}


void flying::draw(int i) 
{
  // turning the propellas
  if (bigProp >= 360) bigProp =0.0f;
  bigProp = bigProp + 25.0;

  if (i == 1) {
    glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
    glCallList(robotList);
    glRotatef(bigProp, 0.0f, 1.0f, 0.0f);
    glCallList(bigRotorList);
  }
  if (placed == 1 && i == 0) {
    glPushMatrix();
      glTranslatef(pos[0], pos[1], pos[2]);
      glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
      glCallList(robotList);
      glRotatef(bigProp, 0.0f, 1.0f, 0.0f);
      glCallList(bigRotorList);
    glPopMatrix();
  }
}







