#ifndef _RRRRRT_H_ 
#define _RRRRRT_H_
/*
Com S 476/576 Course Software 
Release 0.1  02/22/00

Copyright 1999,2000

Steven M. LaValle, Iowa State University
*/
#include <LEDA/plane_alg.h>
#include <LEDA/file.h>
#include <LEDA/matrix.h>
#include <LEDA/graph.h>
#include <LEDA/list.h>
#include <LEDA/ps_file.h>
#include <LEDA/vector.h>

// The RRT Base Class
class RRT {
 protected:
  list<node> PathToLeaf(const node &n, const GRAPH<vector,vector> &g);
  vector RandomState();
  virtual vector ChooseState();
  random_source R;

  // Find the best input to move from x1 to x2;  nx_best is the new state
  virtual vector SelectBestInput(const vector &x1, const vector &x2, 
			 vector &nx_best, bool &success, bool forward);

  // Find the nearest neighbor in g to the state x
  virtual node NearestNeighbor(const vector &x, const GRAPH<vector,vector> &g);

  // Extend the RRT by taking a small step toward the random sample
  virtual bool Extend(const vector &x, GRAPH<vector,vector> &g, node &nn,
		      bool forward);

  // Extend the RRT by attempting to reach the random sample
  virtual bool Connect(const vector &x, GRAPH<vector,vector> &g, node &nn,
		       bool forward);

 public:
  double GoalDist;  // How far is the tree from the goal?

  // An RRT graph:  vertices are states, edges are inputs
  GRAPH<vector,vector> G;

  // A second graph, used for bidirectional planning
  GRAPH<vector,vector> GG;  // RRT from Goal (G is from init)

  // Total number of nodes
  int NumNodes;

  // Maximum allowable number of nodes
  int MaxNodes;

  double DeltaT;

  // This is essentially the "input" to the planner
  Problem *P;

  RRT(Problem *problem);
  virtual ~RRT() {};

  // Construct an RRT
  virtual void Explore();

  // Attempt to find a path from the initial to the goal
  virtual bool Plan(double gdist,list<node> &path);

  // Read/write G and GG to and from a file
  void WriteRRT(const string &fname);
  void ReadRRT(const string &fname);
};



class RRTGoalBias: public RRT {
 public:
  double GoalProb;  // The probability that the goal is selected
  RRTGoalBias(Problem *p);
  virtual ~RRTGoalBias() {};
  virtual vector ChooseState();
};


// Use Connect to grow the RRT for Explore
class RRTCon: public RRT {
 public:
  RRTCon(Problem *p);
  virtual ~RRTCon() {};
  virtual void Explore();
};


// A bidirectional planner
// Use Extend to grow each RRT
// Use Extend to grow the RRTs into each other
class RRTExtExt: public RRT {
 public:
  RRTExtExt(Problem *p);
  virtual ~RRTExtExt() {};
  virtual bool Plan(double gdist,list<node> &path);
};


// A bidirectional planner
// Use Extend to grow each RRT
// Use Connect to grow the RRTs into each other
class RRTExtCon: public RRT {
 public:
  RRTExtCon(Problem *p);
  virtual ~RRTExtCon() {};
  virtual bool Plan(double gdist,list<node> &path);
};


// A bidirectional planner
// Use Connect to grow each RRT
// Use Connect to grow the RRTs into each other
class RRTConCon: public RRT {
 public:
  RRTConCon(Problem *p);
  virtual ~RRTConCon() {};
  virtual bool Plan(double gdist,list<node> &path);
};
#endif


