/*
  simulator.cpp

  Frode Aarstad, 04.19.2000
 */

#include "simulator.h"


simulator::simulator(RRT *r)
{
  planner = r;
  frame = 0;
}


simulator::~simulator()
{
  delete planner;
}


void simulator::drawPath(list<node> nodes, string color)
{
  node n;
  vector x_old(3);
  vector x_new(3);

  if (nodes.size() > 0) {
    // we have found a valid path.
    int fir= 1;
    forall(n, nodes) {
      glPushMatrix();

      if (planner->G.inf(n).dim() == 2) { 
	x_old[0] = (planner->G.inf(n))[0];
	x_old[1] = 0.2;
        x_old[2] = (planner->G.inf(n))[1];
      }
      else {
	x_old[0] = (planner->G.inf(n))[0];
	x_old[1] = (planner->G.inf(n))[1];
	x_old[2] = (planner->G.inf(n))[2];
	//cout << "node3 :"<<(planner->G.inf(n))[3]<<endl;
      }

      glTranslatef(x_old[0], x_old[1], x_old[2]);
      
      if (color == "red") 
	glColor3f(1.0, 0.0, 0.1);
      else if (color == "yellow")
	glColor3f(1.0, 1.0, 0.0);
      else if (color =="blue")
	glColor3f(0.0, 0.1, 1.0);
      else if (color =="white")
       glColor3f(1.0, 1.0, 1.0);

      glutSolidSphere(0.1, 6, 6);
      glPopMatrix();

      if (fir != 1) {
      glPushMatrix();
      glBegin(GL_LINES);
       glVertex3f(x_old[0], x_old[1], x_old[2]);
       glVertex3f(x_new[0], x_new[1], x_new[2]);
      glEnd();
      glPopMatrix();
      }
      fir = 0;
      x_new = x_old;
    }
  }
} // end drawPath(). 
  



void simulator::drawNodes(string color)
{
  node n;
  vector q;

  if (planner->G.number_of_nodes() > 0) {
    forall_nodes(n,planner->G) {
      q = planner->G.inf(n);
      glPushMatrix();
      if (q.dim() == 2)
	glTranslatef(q[0], 0.2, q[1]);
      else {
	glTranslatef(q[0], q[1], q[2]);
      }
      if (color == "red") 
	glColor3f(1.0, 0.0, 0.1);
      else if (color == "yellow")
	glColor3f(1.0, 1.0, 0.0);
      else if (color =="blue")
	glColor3f(0.0, 0.1, 1.0);
      else if (color =="white")
       glColor3f(1.0, 1.0, 1.0);
      
      glutSolidSphere(0.05, 6, 6);
      glPopMatrix();
    }
  }
} //end drawNodes()



void simulator::simulatePath(list<node> nodes, robot *rob)
{
  node n;
  vector pos;
  vector prev_pos;

  if (nodes.size() > 0 && frame != INF) {
    n= nodes.inf(nodes[frame]);
    pos = planner->G.inf(n);
    glPushMatrix();
    
    if (pos.dim() == 2) {
      glTranslatef(pos[0], 0.1, pos[1]);
      rob->draw(1);
    }
    else {
      glTranslatef(pos[0], pos[1], pos[2]);
      glRotatef((pos[3]*360)/2.0*PI, 0.0, 1.0, 0.0);
      rob->draw(1);
    }
    leda_wait(0.5);
    glPopMatrix();
    if (frame < nodes.size()-1) frame++;
  }
}










