#ifndef _WORLD_H_ 
#define _WORLD_H_
/*
Com S 476/576 Course Software 
Release 0.1  02/22/00

Copyright 1999,2000

Steven M. LaValle, Iowa State University
*/

#include <LEDA/file.h>
#include <LEDA/array.h>
#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>
#include <LEDA/graph.h>


#include "maze3D.h"
#include "robot.h"

// The world places objects using configuration (model uses state).
// Each derived class can define Obst and Robot however it likes.
// Methods such as "WorldToPolygons" can give output in whatever 
//  format is requested.  So far, only list<polygon> is shown,
//  but we could imagine an OpenInventor model, Geomview model, etc.
class World {
 protected:
  string FilePath;
 public:
  World(string path) {};
  virtual ~World() {};
  virtual bool CollisionFree(const vector &q) = 0; // Input is configuration
  virtual double DistanceComp(const vector &q) = 0;  // Distance in world
  virtual vector LinearInterpolate(const vector &q1, const vector &q2, 
				   const double &a);  // Depends on topology
  virtual list<polygon> EnvironmentToPolygons() = 0;
  virtual list<polygon> RobotToPolygons(const vector &q) = 0;
};


// Parent class for all 2D polygonal world problems.
// Methods here assume a point robot.  Derived classes assume more
// complicated robots. 
class WorldPolygonal: public World {
 protected:
  list<polygon> Obst;
 public:
  WorldPolygonal(string path);
  virtual ~WorldPolygonal() {};
  virtual bool CollisionFree(const vector &q); // Input is configuration
  virtual double DistanceComp(const vector &q);  // Distance in world
  // These two functions enable a "frame" to be displayed in using
  // LEDA polygons, even if Obst and Robot are represented using
  // other (including 3D) primitives).
  virtual list<polygon> EnvironmentToPolygons();
  virtual list<polygon> RobotToPolygons(const vector &q);
  // Other methods could be added here that are similar to the two
  // above, but output the world in different formats, such 
  // as: Geomview file, Open Inventor file, GL primitives, etc.
};



class WorldPolygonalRigid: public WorldPolygonal {
 private:
  list<polygon> Robot;
 public:
  WorldPolygonalRigid(string path);
  virtual ~WorldPolygonalRigid() {};
  virtual bool CollisionFree(const vector &q); // Input is configuration
  virtual double DistanceComp(const vector &q);  // Distance in world
  virtual list<polygon> RobotToPolygons(const vector &q);
  virtual vector LinearInterpolate(const vector &q1, const vector &q2, 
				   const double &a);  // Depends on topology
};



class worldPQP: public World {
public:
  worldPQP(maze3D *,robot *, string);
  virtual ~worldPQP();

  virtual bool CollisionFree(const vector &q);
  virtual double DistanceComp(const vector &q);
  virtual vector LinearInterpolate(const vector &q1, const vector &q2, 
				   const double &a);

  // not implemented
  virtual list<polygon> EnvironmentToPolygons();
  virtual list<polygon> RobotToPolygons(const vector &q);

 
private:
  maze3D *theMaze;
  robot  *theRobot;
};
#endif











